from flask_restplus import Resource

from portal import facade
from portal.database.models import Client
from portal.facade.courses_facade import CoursesFacade
from portal.facade.users_facade import UsersFacade
from portal.service.auth import AuthService
from portal.service.find import FindService
from portal.service.permissions import PermissionsService
from portal.service.rest import RestService


class FacadesHolder:
    def __init__(self):
        self._users = facade.UsersFacade()
        self._courses = facade.CoursesFacade()
        self._roles = facade.RolesFacade()
        self._groups = facade.GroupsFacade()
        self._secrets = facade.SecretsFacade()
        self._workers = facade.WorkersFacade()
        self._submissions = facade.SubmissionsFacade()

    @property
    def users(self) -> UsersFacade:
        return self._users

    @property
    def courses(self) -> CoursesFacade:
        return self._courses


class CustomResource(Resource):
    @property
    def rest(self) -> RestService:
        if not hasattr(self, '__rest_service'):
            setattr(self, '__rest_service', RestService())
        return getattr(self, '__rest_service')

    @property
    def facades(self) -> FacadesHolder:
        if not hasattr(self, '__facades'):
            setattr(self, '__facades', FacadesHolder())
        return getattr(self, '__facades')

    @property
    def find(self) -> FindService:
        if not hasattr(self, '__find_service'):
            setattr(self, '__find_service', FindService())
        return getattr(self, '__find_service')

    @property
    def permissions(self) -> PermissionsService:
        if not hasattr(self, '__perm_service'):
            setattr(self, '__perm_service', PermissionsService())
        return getattr(self, '__perm_service')

    @property
    def auth(self) -> AuthService:
        if not hasattr(self, '__auth_service'):
            setattr(self, '__auth_service', AuthService())
        return getattr(self, '__auth_service')

    @property
    def client(self) -> Client:
        return self.auth.client
